/* A list of integers which can be appended to and automatically allocates
 * memory.
 * 
 * ILIST.element[i]          The i'th element in the list
 * ILIST.elements            The number of elements in the list 
 * ILIST.size                The amount of memory (in elements) allocated
 */
typedef struct{
  int *element;
  int elements;
  int size;
} ILIST;


/* A list of doubles which can be appended to and automatically allocates
 * memory.
 * 
 * DLIST.element[i]          The i'th element in the list
 * DLIST.elements            The number of elements in the list 
 * DLIST.size                The amount of memory (in elements) allocated
 */
typedef struct{
  double *element;
  int elements;
  int size;
} DLIST;


/* Information about the option at a specific point in price-time space.
 *
 * POINT.value               The value of the option
 * POINT.value_if_held       The expected return from holding the option
 * POINT.value_if_exercised  The payoff if the option is exercised immediately
 * POINT.important_point     A list of points at the next timeslice to be used
 *                             in mesh estimates for this point
 * POINT.g_partial           This points height in the distribution of points
 *                             at this timeslice.  Sum of the transition 
 *                             probabilities from all points at timeslice -1
 *                             to this point
 * POINT.price[i]            The price vector for this point
 * POINT.error               The error in the mesh estimate of
 *                             POINT.value_if_exercised
 * POINT.exact_control       A list of the exact values for the inner control
 *                             for each point in the important_point list
 * POINT.estimated_control   A list of the mesh estimates of the inner control
                               for each point in the important_point list 
 * POINT.delta_r             value_if_held - value_if_exercised
 * POINT.date                How far into the future from the initial time
 *                             this point is
 * POINT.children            How many points at t+1 were created from this point.
 */
typedef struct{
  double value;
  double value_if_held;
  double value_if_exercised;
  ILIST important_point;
  double g_partial;
  double *price;
  double error;
  DLIST exact_control;
  DLIST estimated_control;
  double delta_r;
  double date;
  int children;
} POINT;


/* A list of POINTs which can be appended to and automatically allocates
 * memory.
 * 
 * PLIST.element[i]          The i'th element in the list
 * PLIST.elements            The number of elements in the list 
 * PLIST.size                The amount of memory (in elements) allocated
 */
typedef struct{
  POINT *element;
  int elements;
  int size;
} PLIST;

