#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "global.h"
#include "functions.h"
#include "util.h"
#include "ranlib.h"
#include "our_getopt.h"

/* Global variables from our_getopt.c */
extern char *optarg;
extern int optind;

void option_error(char *argv[]){
  printf("%s: option_error\n",argv[0]);
  printf("****PARAMETERS****\n");
  printf("d+1 Timeslices          \n");
  printf("n   Dimensions          \n");
  printf("S   Initial_price       \n");
  printf("T   Expiration          \n");
  printf("r   Interest_rate       \n");
  printf("D   Dividend_rate       \n");
  printf("t   Trace               \n");
  printf("e   Error_target        \n");
  printf("F   Error_floor         \n");
  printf("C   Error_ceiling       \n");
  printf("p   Columbia_paths      \n");
  printf("K   Strike_price        \n");
  printf("I   which_inner_control \n");
  printf("R   Relevance_cutoff    \n");
  printf("s   Simulation_type     \n");
  exit(1);

}

void handle_command_line(int argc, char *argv[]){
  int n,T,d,I,option,i,j,p,t,s;
  double r,D,v,K,S,e,F,C,R;
  char *options = "n:r:D:v:T:K:S:d:I:e:F:C:p:s:t";

  /* Defaults */
  n = 1;                     /* Number of underliers */
  r = 0.05;                  /* Interest rate */
  D = 0;                     /* Dividend rate */
  v = 0.04;                  /* Variance */
  T = 3;                     /* Time to expiration */
  K = 100;                   /* Strike price */
  S = 90;                    /* Initial price */
  d = 1;                     /* Timeslices (0 - d) */
  I = 0;                     /* Inner Control */
  e = 3;                     /* Error target. Require error to be less than
                                e*delta_r */
  F = 0.0001;                /* Error requirement floor. Never require
                                relative error to be less than F, no matter 
                                e and delta_r */
  C = 0.01;                  /* Error requirement ceiling. Never allow error
                                relative greater than C, no matter e and
                                delta_r */
  t = 0;                     /* Trace: -t means trace with default trace filename.
                                -t filename means trace with given filename. */
  p = 50;                     /* Mesh paths for columbia emulation.  If it is non-zero
                                 we are emulating. */
  s = 2;                     /* Simulation type: 0=columbia, 1=tree, 2=npac */
  R = 0.01;                  /* Relevance_cutoff */

  /* - - - - - Read command line parameters - - - - - */
  while((option = our_getopt(argc,argv,options)) != EOF){
    switch(option){
    case('R'):
      sscanf(optarg,"%lf",&R);
      break;
    case('s'):
      sscanf(optarg,"%i",&s);
    case('p'):
      sscanf(optarg,"%i",&p);
      break;
    case('C'):
      sscanf(optarg,"%lf",&C);
      break;
    case('e'):
      sscanf(optarg,"%lf",&e);
      break;
    case('F'):
      sscanf(optarg,"%lf",&F);
      break;
    case('I'):  
      sscanf(optarg,"%i",&I);
      break;
    case('n'):  
      sscanf(optarg,"%i",&n);
      break;
    case('r'):
      sscanf(optarg,"%lf",&r);
      break;
    case('D'):
      sscanf(optarg,"%lf",&D);
      break;
    case('v'):
      sscanf(optarg,"%lf",&v);
      break;
    case('T'):
      sscanf(optarg,"%lf",&T);
      break;
    case('K'):
      sscanf(optarg,"%lf",&K);
      break;
    case('S'):
      sscanf(optarg,"%lf",&S);
      break;
    case('d'):
      sscanf(optarg,"%i",&d);
      break;
    case('t'):
      t = 1;
      Trace_file = "default_trace_file";
      break;
    default:
      option_error(argv);
    }
  }

  allocate1dDouble(&Mean,n);
  allocate1dDouble(&Dividend_rate,n);
  allocate1dDouble(&Dividend_adjusted_mean,n);
  allocate2dDouble(&Covariance,n,n);

  Timeslices = d+1;
  Dimensions = n;
  Initial_price = S;
  Expiration = T;
  Interest_rate = r;

  for(i=0; i<n; i++){
    Mean[i] = r*T/d;
    Dividend_rate[i] = D*t/d;
    Dividend_adjusted_mean[i] = (r - D)*T/d;
  }
  for(i=0; i<n; i++){
    for(j=0; j<n; j++){
      Covariance[i][j] = (i==j)?v*T/d:0;
    }
  }
  Trace = t;
  Trace_level = 0;
  Error_target = e;
  Error_floor = F;
  Error_ceiling = C;
  Columbia_paths = p;
  Strike_price = K;
  which_inner_control = I;
  Relevance_cutoff = R;
  Simulation_type = s;

  printf("****PARAMETERS****\n");
  printf("d+1 Timeslices          %i\n",Timeslices);
  printf("n   Dimensions          %i\n",Dimensions);
  printf("S   Initial_price       %lf\n",Initial_price);
  printf("T   Expiration          %lf\n",Expiration);
  printf("r   Interest_rate       %lf\n",Interest_rate);
  printf("D   Dividend_rate       %lf\n",Dividend_rate[0]);
  printf("t   Trace_file          %s\n",Trace_file);
  printf("e   Error_target        %lf\n",Error_target);
  printf("F   Error_floor         %lf\n",Error_floor);
  printf("C   Error_ceiling       %lf\n",Error_ceiling);
  printf("p   Columbia_paths      %i\n",Columbia_paths);
  printf("K   Strike_price        %lf\n",Strike_price);
  printf("I   which_inner_control %i\n",which_inner_control);
  printf("R   Relevance_cutoff    %lf\n",Relevance_cutoff);
  printf("s   Simulation_type     %i\n",Simulation_type);
}
