/************************************************************************
 lists.c

 Routines to create, append to and extend integer, double and point lists.
 ************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include "data_structure.h"
#include "global.h"
#include "functions.h"
#include "util.h"


/* - - - - - - - - - -
 * Routine to allocate ilist 
 * Arguments: ILIST *ilist.
 * Allocates memory for ilist and intiializes some of the arays.
 */
int ilist_allocate(ILIST *ilist){
  ilist = (ILIST *)malloc(sizeof(ILIST) );
  if(ilist == NULL){
    printf("ilist_allocate: Could allocate memory\n");
    return(-1);
  }
  ilist->size = 0;
  ilist->elements =0;
  return(0);
} /* end ilist_allocate */

/* - - - - - - - - - -
 * Routine to deallocate ilist 
 * Arguments: ILIST *ilist.
 * Deallocates memory for ilist 
 */
int ilist_deallocate(ILIST *ilist){
  if(ilist->element)free(ilist->element);
  free(ilist);
  return(0);
} /* end ilist_allocate */

/*  - - - - - - - - - -
 *  Routine to extend an existing integer list  
 *  Arguments: ILIST *ilist. 
 *  Increases the length of the elements array by EXTENSION elements.
 *  EXTENSION is defined in data_struct.h
 */
int ilist_extend(ILIST *ilist){
   /* add EXTENSION number of elements (set in functions.h)*/
   ilist->size += EXTENSION; 
   
   /* allocate additional memory */
   ilist->element = (int *)realloc(ilist->element,sizeof(int)*ilist->size);
   if(ilist->element == NULL){
     printf("ilist_extend: Could not allocate memory\n");
     return(-1);
   }
   return(0);
} /* end ilist_extend */

/*  - - - - - - - - - -
 *  Routine to append to integer list 
 *  Arguments: ILIST *ilist and int value
 *  Adds element contained  in value to the end of the list.
 *  If the list is not long enough it extends it.
 */
int ilist_append(ILIST *ilist, int value){
   int error_code;
   /* first check to make sure there is enough space */
   if(ilist->elements ==  ilist->size){/* if it is greater we are screwed */
     error_code = ilist_extend(ilist);
     if(error_code){
       printf("ilist_append: Cannot append value\n");
       return(-1);
     }
   }
   /* now add to end of list */
   ilist->elements++;
   ilist->element[ilist->elements - 1] = value;
    return(0);
  
} /* end ilist_append */

/*  - - - - - - - - - -
 *  Routine to copy ilists 
 *  Arguments: ILIST *ilist. 
 *  Increases the length of the elements array by EXTENSION elements.
 *  EXTENSION is defined in data_struct.h
 */
int ilist_copy(ILIST *from, ILIST *to){
  int i, error_code;
 
  /* First check to make sure pointers are allocated */
  if ( !from || !to){
    printf("ilist_copy: from or to pointer not initialized\n");
    return(-1);
  }
  error_code = 0;
  for(i = 0; i < from->elements; i++){
    error_code += ilist_append(to, from->element[i]);
  }

  return(error_code);
} /* end ilist_copy */

/* - - - - - - - - - -
 * Routine to allocate dlist 
 * Arguments: DLIST *dlist.
 * Allocates memory for dlist and intiializes some of the arays.
 */
int dlist_allocate(DLIST *dlist){
  dlist = (DLIST *)malloc(sizeof(DLIST) );
  if(dlist == NULL){
    printf("dlist_allocate: Could allocate memory\n");
    return(-1);
  }
  dlist->size = 0;
  dlist->elements =0;
  return(0);
} /* end dlist_allocate */

/* - - - - - - - - - -
 * Routine to deallocate dlist 
 * Arguments: DLIST *dlist.
 * Deallocates memory for dlist 
 */
int dlist_deallocate(DLIST *dlist){
  if(dlist->element)free(dlist->element);
  free(dlist);
  return(0);
} /* end dlist_allocate */

/*  - - - - - - - - - -
 *  Routine to extend an existing double list 
 *  Arguments: DLIST *dlist. 
 *  Increases the length of the elements array by EXTENSION elements.
 *  EXTENSION is defined in data_struct.h
 */
int dlist_extend(DLIST *dlist){
   /* add EXTENSION number of elements (set in functions.h)*/
   dlist->size += EXTENSION; 
   
   /* allocate additional memory */
   dlist->element = (double *)realloc(dlist->element, 
                    sizeof(double)*dlist->size);
   if(dlist->element == NULL){
     printf("dlist_extend: Could not allocate memory\n");
     return(-1);
   }
   return(0);
} /* end dlist_extend */

/*  - - - - - - - - - -
 *  Routine to append to double list 
 *  Arguments: DLIST *dlist and double value
 *  Adds element contained  in value to the end of the list.
 *  If the list is not long enough it extends it.
 */
int dlist_append(DLIST *dlist, double value){
   int error_code;
   /* first check to make sure there is enough space */
   if(dlist->elements ==  dlist->size){/* if it is greater we are screwed */
     error_code = dlist_extend(dlist);
     if(error_code){
       printf("dlist_append: Cannot append value\n");
       return(-1);
     }
   }
   /* now add to end of list */
   dlist->elements++;
   dlist->element[dlist->elements - 1] = value;
    return(0);
  
} /* end dlist_append */
/*  - - - - - - - - - -
 *  Routine to copy dlists 
 *  Arguments: DLIST *dlist. 
 *  Increases the length of the elements array by EXTENSION elements.
 *  EXTENSION is defined in data_struct.h
 */
int dlist_copy(DLIST *from, DLIST *to){
  int i, error_code;
 
  /* First check to make sure pointers are allocated */
  if ( !from || !to){
    printf("dlist_copy: from or to pointer not initialized\n");
    return(-1);
  }
  error_code = 0;
  for(i = 0; i < from->elements; i++){
    error_code += dlist_append(to, from->element[i]);
  }

  return(error_code);
} /* end dlist_copy */

/* - - - - - - - - - -
 * Routine to allocate point 
 * Arguments: POINT *point.
 * Allocates memory for point and intiializes some of the arays.
 */
int point_allocate(POINT *point){
  int error_code;
  error_code = 0;
  /* Allocate memory*/
  point = (POINT *)malloc(sizeof(POINT));
  if(point == NULL){
    printf("point_allocate: Could not allocate memory\n");
    return(-1);
  }

  /* place to initialize values...  */
  point->value = 0.0;
  point->value_if_held = 0.0;
  point->value_if_exercised = 0.0;
  error_code += ilist_allocate(&(point->important_point));
  point->g_partial = 0;
  point->price = (double *)malloc(sizeof(double) * Dimensions);
  if(point->price == NULL){
    printf("point_allocate: Could not allocate price array\n");
    return(-1);
  }
  point->error = 0.0;
  error_code += dlist_allocate(&(point->exact_control));
  error_code += dlist_allocate(&(point->estimated_control));
  point->delta_r = 0.0;
  point->date = 0.0;   /* needs initialization */

  return(error_code);
} /* end point_allocate */

/* - - - - - - - - - -
 * Routine to deallocate point 
 * Arguments: POINT *point.
 * Allocates memory for point and intiializes some of the arays.
 */
int point_deallocate(POINT *point){
  int error_code;
  error_code = 0;

  /* place to initialize values...  */
  error_code += ilist_deallocate(&(point->important_point));
  free(point->price);
  error_code += dlist_deallocate(&(point->exact_control));
  error_code += dlist_deallocate(&(point->estimated_control));

  free(point);

  return(error_code);
} /* end point_deallocate */

/*  - - - - - - - - - -
 *  Routine to extend list of points 
 *  Arguments: PLIST *plist. 
 *  Increases the length of the elements array by EXTENSION elements.
 *  EXTENSION is defined in data_struct.h
 */
int plist_extend(PLIST *plist){
   /* add EXTENSION number of elements (set in functions.h)*/
   plist->size += EXTENSION; 
   
   /* allocate additional memory */
   plist->element = (POINT *)realloc(plist->element, 
                    sizeof(POINT)*plist->size);
   if(plist->element == NULL){
     printf("plist_extend: Could not allocate memory\n");
     return(-1);
   }
   return(0);
} /* end plist_extend */
/* - - - - - - - - - -
 * Routine to allocate plist 
 * Arguments: PLIST *plist.
 * Allocates memory for plist and intiializes some of the arays.
 */
int plist_allocate(PLIST *plist){
  plist = (PLIST *)malloc(sizeof(PLIST) );
  if(plist == NULL){
    printf("plist_allocate: Could allocate memory\n");
    return(-1);
  }
  plist->size = 0;
  plist->elements =0;
  return(0);
} /* end plist_allocate */

/* - - - - - - - - - -
 * Routine to deallocate plist 
 * Arguments: PLIST *plist.
 * Deallocates memory for plist 
 */
int plist_deallocate(PLIST *plist){
  if(plist->element)free(plist->element);
  free(plist);
  return(0);
} /* end plist_allocate */

/* - - - - - - - - - -
 * Routine to copy plist's 
 * Arguments: PLIST *from, PLIST *to.
 * Deallocates memory for plist 
 */
int plist_copy(PLIST *from, PLIST *to){
  int i, error_code;
 
  /* First check to make sure pointers are allocated */
  if ( !from || !to){
    printf("plist_copy: from or to pointer not initialized\n");
    return(-1);
  }
  error_code = 0;
  for(i = 0; i < from->elements; i++){
    error_code += plist_append(to, &(from->element[i]));
  }

  return(error_code);
} /* end plist_copy */


/* - - - - - - - - - - 
 * Routine to copy points 
 */
void point_copy( POINT *from , POINT *to){
  int i;
  from->value = to->value;
  from->value_if_held = to->value_if_held;
  from->value_if_exercised = to->value_if_exercised;
  ilist_copy(&(from->important_point), &(to->important_point));
  from->g_partial = to->g_partial;
  for(i=0;i<Dimensions; i++){
    from->price[i] = to->price[i];
  }
  from->error = to->error;
  dlist_copy(&(from->exact_control), &(to->exact_control));
  dlist_copy(&(from->estimated_control), &(to->estimated_control));
  from->delta_r = to->delta_r;
  from->date = to->date;
} /* end point_copy */

/*  - - - - - - - - - -
 *  Routine to append to list of points  
 *  Arguments: PLIST *plist and double value
 *  Adds element contained  in value to the end of the list.
 *  If the list is not long enough it extends it.
 */
int plist_append(PLIST *plist, POINT *value){
   int error_code;
   /* first check to make sure there is enough space */
   if(plist->elements ==  plist->size){/* if it is greater we are screwed */
     error_code = plist_extend(plist);
     if(error_code){
       printf("plist_append: Cannot append value\n");
       return(-1);
     }
   }
   /* now add to end of list */
   plist->elements++;
   allocate_point(&(plist->element[plist->elements - 1]));
   point_copy(&(plist->element[plist->elements - 1]) , value);
    return(0);
  
} /* end plist_append */


