/* 
  mesh.c
  Contains routines for making mesh estimates and mesh estimates of 
  the error.
*/
#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include "functions.h"
#include "global.h"
#include "util.h"
#include "plist.h"
#include "blackScholes.h"

/* Prototypes for mesh estimator */
double controlledMeshEstimate0(POINT *point, int timeslice, double *variance);
double controlledMeshEstimate3(POINT *point, int timeslice, double *variance);

/*  - - - - - - - - - -
   Routine to compute mesh estimate.
   Uses Time and which_inner_control from global.h
*/
double mesh_estimate_value(POINT *point, int timeslice, double *error){

/* printf("old estimate: %lf\n",point->value_if_held);	 */
  /* 
   * Compute the mesh estimate of the value and error.
   * Use inner control specified in which_inner_control in global.h
   */
  switch(which_inner_control){
  case(3):
    return(controlledMeshEstimate3(point, timeslice,error));
    break;
  default:
    return(controlledMeshEstimate0(point, timeslice,error));
    break;
  } /* end switch on which_inner_control */

} /* end mesh_estimate_value */

/*  - - - - - - - - - -
 *  Routine to compute the weight.
 *  Assumption is that g_partial is normalized at this point.
 */
double weight(POINT *from, POINT *to, double g_partial){
  return(transition_probability(from, to) / g_partial);
}


/* 
 * Routine to compute the mesh estimate of the value and error without
 * inner controls.
 *
 */
double controlledMeshEstimate0(POINT *point, int timeslice, double *variance){
  int toPath, i;
  POINT *this;
  double f,w,S1,S2,S3,S4,S5,S6;
  double sumg;

  S1 = 0;
  S2 = 0;
  S3 = 0;
  S4 = 0;
  S5 = 0;
  S6 = 0;
  this = Time[timeslice+1].element;  /* pick out pointer to PLIST */

  /* Compute the normalization of g from teh list of important points */
  sumg = 0.0;
  for(i=0; i< point->important_point.elements; i++){
    toPath =  point->important_point.element[i];
    sumg += this[toPath].g_partial;
    /* printf("meshEst0: toPath = %d g = %lf sumg = %lf\n",
        toPath,  this[toPath].g_partial, sumg);				 */
  }

 

  /* Loop over important points and construct mesh estimates and error */
  for(i=0; i< point->important_point.elements; i++){
    toPath =  point->important_point.element[i];
    f = this[toPath].value;
    /* get the weight */
    w = weight(point, &(this[toPath]), this[toPath].g_partial/sumg);
    /*printf("meshEst0: toPath = %d w = %lf\n",toPath, w);
    printf("meshEst0: toPath = %d value at t+1 = %lf\n",toPath, f);			*/
		S1 += w;
    S2 += w*w;
    S3 += w*f;
    S4 += w*w*f;
    S6 += w*w*f*f;
  }
  *variance = (1/(S1*S1))*(S6 - 2*S3/S1*S4 + (S3/S1)*(S3/S1)*S2);

  return(S3/S1);
}


/* 
 * Routine to compute the mesh estimate of the value and the error with
 * inner control 3.
 */
double controlledMeshEstimate3(POINT *point, int timeslice, double *variance){
  int toPath, max1, max2, i;
  double S1, S2, S7, S8, S9 ,S10, S11, S12, S13, w, f, g, G, maxval1, maxval2,
    maxPrice, beta, S14, S15, S16;
  double sumg;
  POINT *this;

  S1 = 0;
  S2 = 0;
  S7 = 0;
  S8 = 0;
  S9 = 0;
  S10 = 0;
  S11 = 0;
  S12 = 0;
  S13 = 0;
  S14 = 0;
  S15 = 0;
  S16 = 0;

  /* At the given time find the max prices */

  maxval1 = 0.0;
  maxval2 = 0.0;
  max1 = dWhichMax(Dimensions, point->price);
  maxval1 = point->price[max1];

  for(i=0; i<Dimensions;i++){
    if(i != max1 && point->price[i] > maxval2){
      max2 = i;
      maxval2 = point->price[i];
    }
  }

  /***** IMPORTANT NOTE *****
   HAD TO CHANGE Interest_rate to Dividend_adjusted_mean[0] to get
   identical to mesh2!! 
   THIS IS TAKING THE DIVIDEND INTO ACCOUNT TWICE AS IS MESH2 
   Needs work here!! 
   Substitute Dividend_adjusted_mean[0] by Interest_rate in all the
   lines in this routine below...
  */
  G = exp(-timeslice*Dividend_adjusted_mean[0])* 
    maxOf2Risk(maxval1, maxval2, Strike_price, 1.0,
               sqrt(Covariance[max1][max1]), 
               sqrt(Covariance[max2][max2]), 
               Dividend_adjusted_mean[0], 
               Dividend_rate[max1], 
               Dividend_rate[max2],
               Covariance[max1][max2]);

  /* pick out pointer to data at next timeslice */
  this = Time[timeslice+1].element;

  /* Compute the normalization of g from teh list of important points */
  sumg = 0.0;
  for(i=0; i< point->important_point.elements; i++){
    toPath =  point->important_point.element[i];
    sumg += this[toPath].g_partial;
  }

  /* Loop over important points and construct mesh estimates and error */
  for(i=0; i< point->important_point.elements; i++){
    toPath =  point->important_point.element[i];
    f = this[toPath].value;
    /* get the weight */
    w = weight(point, &(this[toPath]), this[toPath].g_partial/sumg);

    if( this[toPath].price[max1] >= this[toPath].price[max2]){
      maxPrice = this[toPath].price[max1];
    }else{
      maxPrice = this[toPath].price[max2];
    }
    if(maxPrice > Strike_price){
      g = exp(-(timeslice+1)*Dividend_adjusted_mean[0])*(maxPrice - Strike_price);
    }else{
      g= 0;
    }

    S1 += w;
    S2 += w*w;
    S7 += w*f;
    S8 += w*(g-G);
    S9 += w*w*f;
    S10 += w*w*(g-G);
    S11 += w*w*f*f;
    S12 += w*w*f*(g-G);
    S13 += w*w*(g-G)*(g-G);
    S14 += w*f*(g-G);
    S15 += w*(g-G)*(g-G);
    S16 += w*f*f;
  }

  /* Old derivation */
  beta = ((S14/S1) - (S7/S1)*(S8/S1))/
    ((S15/S1) - (S8/S1)*(S8/S1));

/*   *variance = S16/S1 - (S7/S1)*(S7/S1)   */
/*     -2*beta*(S14/S1 - (S7/S1)*(S8/S1))   */
/*     +beta*beta*((S15/S1) - (S8/S1)*(S8/S1));   */


  /* New derivation */
/*     beta = (S12/S1 - (S8*S9 + S7*S10)/(S1*S1) + S2*S7*S8/(S1*S1*S1))/  */
/*       (S13/S1 - 2*S8*S10/(S1*S1) + S2*S8*S8/(S1*S1*S1));   */

  *variance = 1/(S1*S1)*((S11 -2*beta*S12 + beta*beta*S13)
                         -(2/S1)*(S7-beta*S8)*(S9-beta*S10)
                         +(S2/(S1*S1))*(S7*S7 -2*beta*S7*S8 +beta*beta*S8*S8)
                         );

  return( (S7-beta*S8)/S1);

}
