#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "point.h"
#include "global.h"


int point_allocate(POINT **point){
  *point = (POINT *)malloc(sizeof(POINT));
  if(*point == NULL) return(-1);
}


int point_deallocate(POINT *point){
  free(point);
  return(0);
}


int point_initialize(POINT *point){
  int error_code;

  point->value = 0.0;
  point->value_if_held = 0.0;
  point->value_if_exercised = 0.0;
  point->g_partial = 0;
  point->error = 0.0;
  point->delta_r = 0.0;
  point->date = 0.0;
  point->children = 0;

  point->price = (double *)malloc(sizeof(double) * Dimensions);
  if(point->price == NULL) --error_code;
  error_code += dlist_initialize(&(point->exact_control));
  error_code += dlist_initialize(&(point->estimated_control));
  error_code += ilist_initialize(&(point->important_point));

  return(error_code);
}


int point_finalize(POINT *point){
  int error_code = 0;

  free(point->price);
  error_code += dlist_finalize(&(point->exact_control));
  error_code += dlist_finalize(&(point->estimated_control));
  error_code += ilist_finalize(&(point->important_point));

  return(error_code);
}


int point_copy(POINT *from, POINT *to){
  int error_code = 0;

  point_finalize(to);
  point_initialize(to);
  to->value = from->value;
  to->value_if_held = from->value_if_held;
  to->value_if_exercised = from->value_if_exercised;
  to->g_partial = from->g_partial;
  to->error = from->error;
  to->delta_r = from->delta_r;
  to->date = from->date;
  to->children = from->children;

  error_code += ilist_copy(&(from->important_point), &(to->important_point));
  memcpy(to->price, from->price, Dimensions*sizeof(double));
  error_code += dlist_copy(&(from->exact_control), &(to->exact_control));
  error_code += dlist_copy(&(from->estimated_control), &(to->estimated_control));

  return(error_code);
}


POINT *point_clone(POINT *from){
  int i;
  POINT *to;

  point_allocate(&to);
  point_initialize(to);
  to->value = from->value;
  to->value_if_held = from->value_if_held;
  to->value_if_exercised = from->value_if_exercised;
  to->g_partial = from->g_partial;
  to->error = from->error;
  to->delta_r = from->delta_r;
  to->date = from->date;
  to->children = from->children;

  ilist_copy(&(from->important_point), &(to->important_point));
  memcpy(to->price, from->price, Dimensions*sizeof(double));
  dlist_copy(&(from->exact_control), &(to->exact_control));
  dlist_copy(&(from->estimated_control), &(to->estimated_control));
  
  return(to);
}
