/* Information about the option at a specific point in price-time space.
 *
 * POINT.value               The value of the option
 * POINT.value_if_held       The expected return from holding the option
 * POINT.value_if_exercised  The payoff if the option is exercised immediately
 * POINT.important_point     A list of points at the next timeslice to be used
 *                             in mesh estimates for this point
 * POINT.g_partial           This points height in the distribution of points
 *                             at this timeslice.  Sum of the transition 
 *                             probabilities from all points at timeslice -1
 *                             to this point
 * POINT.price[i]            The price vector for this point
 * POINT.error               The error in the mesh estimate of
 *                             POINT.value_if_exercised
 * POINT.exact_control       A list of the exact values for the inner control
 *                             for each point in the important_point list
 * POINT.estimated_control   A list of the mesh estimates of the inner control
                               for each point in the important_point list 
 * POINT.delta_r             value_if_held - value_if_exercised
 * POINT.date                How far into the future from the initial time
 *                             this point is
 * POINT.children            How many points at t+1 were created from this point.
 */

#ifndef ILIST_STRUCTURE
#include "ilist.h"
#endif

#ifndef ILIST_FUNCTIONS
#include "ilist.h"
#endif

#ifndef DLIST_STRUCTURE
#include "dlist.h"
#endif

#ifndef DLIST_FUNCTIONS
#include "dlist.h"
#endif


#ifndef POINT_STRUCTURE
typedef struct{
  double value;
  double value_if_held;
  double value_if_exercised;
  ILIST important_point;
  double g_partial;
  double *price;
  double error;
  DLIST exact_control;
  DLIST estimated_control;
  double delta_r;
  double date;
  int children;
} POINT;
#define POINT_STRUCTURE
#endif


#ifndef POINT_FUNCTIONS
int point_allocate(POINT **point);
int point_deallocate(POINT *point);
int point_initialize(POINT *point);
int point_finalize(POINT *point);
int point_copy(POINT *from , POINT *to);
POINT *point_clone(POINT *from);
#define POINT_FUNCTIONS
#endif

